'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var animationFrameChunkScheduler = typeof window !== 'undefined' &&
    typeof window.requestAnimationFrame === 'function' &&
    typeof window.cancelAnimationFrame === 'function' ?
    {
        request: function (fn) { return window.requestAnimationFrame(fn); },
        cancel: function (token) { return window.cancelAnimationFrame(token); }
    } :
    null;

var idleCallbackChunkScheduler = typeof window !== 'undefined' &&
    typeof window.requestIdleCallback === 'function' &&
    typeof window.cancelIdleCallback === 'function' ?
    {
        request: function (fn) { return window.requestIdleCallback(fn); },
        cancel: function (token) { return window.cancelIdleCallback(token); }
    } :
    null;

var immediateChunkScheduler = typeof setImmediate === 'function' &&
    typeof clearImmediate === 'function' ?
    {
        request: function (fn) { return setImmediate(fn); },
        cancel: function (token) { return clearImmediate(token); }
    } :
    null;

var timeoutChunkScheduler = {
    request: function (fn) { return setTimeout(fn, 0); },
    cancel: function (token) { return clearTimeout(token); }
};

function getChunkScheduler(type) {
    var chunkScheduler = null;
    switch (type) {
        case 'animationFrame':
            chunkScheduler = animationFrameChunkScheduler;
            break;
        case 'idleCallback':
            chunkScheduler = idleCallbackChunkScheduler;
            break;
        case 'immediate':
            chunkScheduler = immediateChunkScheduler;
            break;
        case 'timeout':
            chunkScheduler = timeoutChunkScheduler;
            break;
        case 'auto':
            chunkScheduler =
                idleCallbackChunkScheduler ||
                    animationFrameChunkScheduler ||
                    immediateChunkScheduler;
            break;
        default:
            chunkScheduler = type;
    }
    return chunkScheduler || timeoutChunkScheduler;
}

var now = typeof performance === 'object' && typeof performance.now === 'function' ?
    function () { return performance.now(); } :
    Date.now;

var DEFAULT_CHUNK_BUDGET = 12;
function createTask(_a) {
    var unit = _a.unit, _b = _a.chunkBudget, chunkBudget = _b === void 0 ? DEFAULT_CHUNK_BUDGET : _b, _c = _a.chunkScheduler, chunkSchedulerType = _c === void 0 ? 'auto' : _c;
    var chunkScheduler = getChunkScheduler(chunkSchedulerType);
    var chunkSchedulerToken = null;
    var nextUnit = unit;
    var result;
    var aborted = false;
    var promise = null;
    return {
        run: function () {
            if (promise !== null) {
                return promise;
            }
            return promise = new Promise(function (resolve, reject) {
                chunkSchedulerToken = chunkScheduler.request(function chunk() {
                    var _a;
                    // needed for chunk schedulers without cancellation api
                    if (aborted) {
                        return;
                    }
                    var startTime = now();
                    var restBudget = chunkBudget;
                    while (nextUnit !== null) {
                        try {
                            (_a = unit(result), nextUnit = _a.next, result = _a.result);
                        }
                        catch (e) {
                            reject(e);
                            return;
                        }
                        var elapsedTime = now() - startTime;
                        restBudget -= elapsedTime;
                        if (restBudget < elapsedTime) {
                            break;
                        }
                        startTime += elapsedTime;
                    }
                    if (nextUnit === null) {
                        resolve(result);
                    }
                    else {
                        chunkSchedulerToken = chunkScheduler.request(chunk);
                    }
                });
            });
        },
        abort: function () {
            if (chunkScheduler.cancel && chunkSchedulerToken !== null) {
                chunkScheduler.cancel(chunkSchedulerToken);
                chunkSchedulerToken = null;
            }
            aborted = true;
        }
    };
}

exports.createTask = createTask;
